'-----------------------------------------------------------
' Atmega168 and WIZ810MJ Webserver
' Build for the WIZ810MJ Evaluation Board 1.0 by Thomas Heldt
'-----------------------------------------------------------
' Software Version 1.0
'-----------------------------------------------------------

$regfile = "m168def.dat"
$crystal = 8000000
$baud = 19200                                               'put terminal on 19200 baud

$hwstack = 32
$swstack = 64
$framesize = 32

'Used variables
Dim A(5) As Byte
Dim Wert As Byte
Dim Status As Byte
Dim Oldstatus As Byte
Dim Pointer_rx As Word
Dim Pointer_tx As Word
Dim Tmp_value As Byte
Dim Tmp_str As String * 1
Dim Found_str As String * 2
Dim Highbyte As Byte
Dim Lowbyte As Byte
Dim Sendbuffer As String * 150
Dim Receivebuffer As String * 650
Dim X As Integer
Dim T As Integer
Dim Auth As Bit
Dim Authentification As Byte
Dim Userpass As String * 15
Dim Tmp_auth As String * 15

'Used ports and pins
Wiz5100_cs Alias Portb.2                                    'Chipselect WIZ5100
Wiz5100_int Alias Portb.1                                   'INT of WIZ5100
Led Alias Portc.5                                           'Debug LED

Config Wiz5100_cs = Output
Config Wiz5100_int = Input
Config Led = Output

Print "System start..."

'Configuration of the SPI-bus
Config Spi = Hard , Interrupt = Off , Data Order = Msb , Master = Yes , Polarity = Low , Phase = 0 , Clockrate = 4 , Noss = 0

'Init the spi pins
Spiinit

'Toggle LED 3 times
Reset Led
Wait 1
Set Led
Wait 1
Reset Led
Wait 1
Set Led
Wait 1
Reset Led
Wait 1
Set Led

'Authentification 0=disabled / 1=enabled
Auth = 1
Userpass = "admin:demo"                                     'Max. 15 characters

Const Ip[1] = 192
Const Ip[2] = 168
Const Ip[3] = 178
Const Ip[4] = 99

Const Sn[1] = 255
Const Sn[2] = 255
Const Sn[3] = 255
Const Sn[4] = 0

Const Gw[1] = 192
Const Gw[2] = 168
Const Gw[3] = 178
Const Gw[4] = 1

Const Mac[1] = &H00
Const Mac[2] = &H20
Const Mac[3] = &H18
Const Mac[4] = &HB1
Const Mac[5] = &H15
Const Mac[6] = &H5F

Const Wiz5100_opcode_read = 15
Const Wiz5100_opcode_write = 240

Declare Sub Wiz5100_init
Declare Sub Wiz5100_readvalue(byval Highbyte As Byte , Byval Lowbyte As Byte)
Declare Sub Wiz5100_writevalue(byval Highbyte As Byte , Byval Lowbyte As Byte , Byval Value As Byte)
Declare Sub Wiz5100_receive()
Declare Sub Wiz5100_send()
Declare Sub Wiz5100_disconnect()

Print "Initialize the WIZ810MJ..."

Call Wiz5100_init

Print "Initialization complete..." + Chr(13) + Chr(10)

Print "IP-Adress: ";
Call Wiz5100_readvalue(&H0 , &HF)
Print Wert ; ".";

Call Wiz5100_readvalue(&H0 , &H10)
Print Wert ; ".";

Call Wiz5100_readvalue(&H0 , &H11)
Print Wert ; ".";

Call Wiz5100_readvalue(&H0 , &H12)
Print Wert

Print "Subnetmask: ";
Call Wiz5100_readvalue(&H0 , &H5)
Print Wert ; ".";

Call Wiz5100_readvalue(&H0 , &H6)
Print Wert ; ".";

Call Wiz5100_readvalue(&H0 , &H7)
Print Wert ; ".";

Call Wiz5100_readvalue(&H0 , &H8)
Print Wert

Print "MAC-Adress: ";
Call Wiz5100_readvalue(&H0 , &H9)
Print Hex(wert) ; ":";

Call Wiz5100_readvalue(&H0 , &HA)
Print Hex(wert) ; ":";

Call Wiz5100_readvalue(&H0 , &HB)
Print Hex(wert) ; ":";

Call Wiz5100_readvalue(&H0 , &HC)
Print Hex(wert) ; ":";

Call Wiz5100_readvalue(&H0 , &HD)
Print Hex(wert) ; ":";

Call Wiz5100_readvalue(&H0 , &HE)
Print Hex(wert)

Print "GW-Adress: ";
Call Wiz5100_readvalue(&H0 , &H1)
Print Wert ; ".";

Call Wiz5100_readvalue(&H0 , &H2)
Print Wert ; ".";

Call Wiz5100_readvalue(&H0 , &H3)
Print Wert ; ".";

Call Wiz5100_readvalue(&H0 , &H4)
Print Wert

Print "System ready" + Chr(13) + Chr(10)

Do
   'Get socket status
   Call Wiz5100_readvalue(&H4 , &H3)
   Status = Wert

   If Status <> Oldstatus Then
      Oldstatus = Status
      Select Case Status
         Case &H10
            Print "???"
         Case &H11
            Print "ARP request"
         Case &H21
            Print "ARP request"
         Case &H31
            Print "ARP request"
         Case &H12
            Print "???"
         Case &H14
            Print "Wait for new conection"
         Case &H15
            Print "SYSNSENT"
         Case &H16
            Print "New connection from IP-Adress ";
            Call Wiz5100_readvalue(&H4 , &HC)
            Print Wert ; ".";

            Call Wiz5100_readvalue(&H4 , &HD)
            Print Wert ; ".";

            Call Wiz5100_readvalue(&H4 , &HE)
            Print Wert ; ".";

            Call Wiz5100_readvalue(&H4 , &HF)
            Print Wert
         Case &H17
            Print "Connection etablished"
            Receivebuffer = ""
         Case &H18
            Print "Wait for new connection"
            Call Wiz5100_disconnect()
            Print "Connection closed"
         Case &H1C
            Print "Closing connection"
            Call Wiz5100_disconnect()
            Print "Connection closed"
         Case Else
            Print "Closing connection -> Status: " ; Status
            Call Wiz5100_disconnect()
            Print "Connection closed"
      End Select
   End If

   If Status = &H17 Then
      Call Wiz5100_receive()
   End If
Loop

End

'Close socket, reopen socket and wait for new connection
Sub Wiz5100_disconnect()
   Call Wiz5100_writevalue(&H4 , &H1 , &H0)                 'CLOSE
   Call Wiz5100_writevalue(&H4 , &H1 , &H1)                 'OPEN
   Call Wiz5100_writevalue(&H4 , &H1 , &H2)                 'LISTEN
End Sub

'Send Headers or websites
Sub Wiz5100_send()
   Local Sendsize As Integer , Pos_tx As Word , Freesize As Word , Offset As Word , Tx_wr As Word
   Local Startadress As Word , Offsend As Word , Uppersize As Word , Glaenge As Word , Startpos As Word

   If Authentification > 0 Then
      If Authentification = 1 Then
         Sendbuffer = "HTTP/1.0 401 Unauthorized" + Chr(13) + Chr(10) + "Server: AVR_Small_Webserver" + Chr(13) + Chr(10) + "WWW-Authenticate: Basic realm=" + Chr(34) + "Website" + Chr(34) + Chr(13) + Chr(10) + "Content-Type: text/html" + Chr(13) + Chr(10) + Chr(13) + Chr(10)
         Incr Authentification
      Else
         Sendbuffer = "401 Unauthorized - User=admin Passwort=demo"
         Incr Authentification
      End If

      Sendsize = Len(Sendbuffer)

      Print "Bytes to send: " ; Sendsize

      Freesize1:
         Call Wiz5100_readvalue(&H4 , &H20)
         Freesize = Wert
         Shift , Freesize , Left , 8
         Call Wiz5100_readvalue(&H4 , &H21)
         Freesize = Freesize + Wert

         If Freesize < Sendsize Then
            Print "Freesize!!!"
            Goto Freesize1
         End If

      Call Wiz5100_readvalue(&H4 , &H24)
      Tx_wr = Wert
      Shift , Tx_wr , Left , 8
      Call Wiz5100_readvalue(&H4 , &H25)
      Tx_wr = Tx_wr + Wert

      Startpos = Tx_wr

      Offset = Tx_wr And &H7FF

      Startadress = &H4000 + Offset

      Offsend = Offset + Sendsize

      Pointer_tx = Startadress

      If Offsend > &H800 Then
         Print "Send action 2"

         Uppersize = &H800 - Offset

         For X = 1 To Uppersize
            Highbyte = High(pointer_tx)
            Lowbyte = Low(pointer_tx)
            Tmp_str = Mid(Sendbuffer , X , 1)
            Tmp_value = Asc(tmp_str)
            Call Wiz5100_writevalue(highbyte , Lowbyte , Tmp_value)
            Incr Pointer_tx
            Print Chr(tmp_value);
         Next X

         Pointer_tx = &H4000
         Incr Uppersize

         For X = Uppersize To Sendsize
            Highbyte = High(pointer_tx)
            Lowbyte = Low(pointer_tx)
            Tmp_str = Mid(Sendbuffer , X , 1)
            Tmp_value = Asc(tmp_str)
            Call Wiz5100_writevalue(highbyte , Lowbyte , Tmp_value)
            Incr Pointer_tx
            Print Chr(tmp_value);
         Next X
      Else
         Print "Send action 1"

         For X = 1 To Sendsize
            Highbyte = High(pointer_tx)
            Lowbyte = Low(pointer_tx)
            Tmp_str = Mid(Sendbuffer , X , 1)
            Tmp_value = Asc(tmp_str)
            Call Wiz5100_writevalue(highbyte , Lowbyte , Tmp_value)
            Incr Pointer_tx
            Print Chr(tmp_value);
         Next X
      End If

      Print ""
      Print "Get TX POINTER"

      Call Wiz5100_readvalue(&H4 , &H24)
      Startpos = Wert
      Shift , Startpos , Left , 8
      Call Wiz5100_readvalue(&H4 , &H25)
      Startpos = Startpos + Wert

      Glaenge = Startpos + Sendsize

      Print "Set SIZE to SEND"

      'Set size of data to send
      Highbyte = High(glaenge)
      Call Wiz5100_writevalue(&H4 , &H24 , Highbyte)

      Lowbyte = Low(glaenge)
      Call Wiz5100_writevalue(&H4 , &H25 , Lowbyte)

      Print "Set SEND Flag"

     'Set SEND flag
      Call Wiz5100_writevalue(&H4 , &H1 , &H20)

      If Authentification > 2 Then
         Print "Set DISCON Flag"

         'Set DISCON flag
         Call Wiz5100_writevalue(&H4 , &H1 , &H8)
      End If
   Else
      'Because we have a litte ram space we will split the website in pieces of max. 150 characters
      For T = 1 To 10
         If T = 1 Then
            Sendbuffer = "HTTP/1.0 200 Document follows" + Chr(13) + Chr(10) + "Server: WIZ810MJ AVR Webserver" + Chr(13) + Chr(10) + "Content-Type: text/html" + Chr(13) + Chr(10) + Chr(13) + Chr(10)
         Elseif T = 2 Then
            Sendbuffer = "<html><head><meta http-equiv=" + Chr(34) + "Pragma" + Chr(34) + " Content=" + Chr(34) + "no cache" + Chr(34) + ">"
         Elseif T = 3 Then
            Sendbuffer = "<title>WIZ810MJ AVR Webserver</title></head><body><center>"
         Elseif T = 4 Then
            Sendbuffer = "<font face=" + Chr(34) + "verdana" + Chr(34) + " color=" + Chr(34) + "#000000" + Chr(34) + "><H1>WIZ810MJ AVR Webserver</H1>"
         Elseif T = 5 Then
            Sendbuffer = "<br><br>Die LED ist "
         Elseif T = 6 Then
            If Led = 0 Then
               Sendbuffer = "<font color=" + Chr(34) + "green" + Chr(34) + "><b>AN</b></font><br><br><br><br><form action=" + Chr(34) + "/" + Chr(34) + " method=" + Chr(34) + "POST" + Chr(34) + ">"
            Else
               Sendbuffer = "<font color=" + Chr(34) + "red" + Chr(34) + "><b>AUS</b></font><br><br><br><br><form action=" + Chr(34) + "/" + Chr(34) + " method=" + Chr(34) + "POST" + Chr(34) + ">"
            End If
         Elseif T = 7 Then
            If Led = 0 Then
               Sendbuffer = "<input type=" + Chr(34) + "hidden" + Chr(34) + " name=" + Chr(34) + "LED" + Chr(34) + " value=" + Chr(34) + "1" + Chr(34) + ">"
               Sendbuffer = Sendbuffer + "<input Type = " + Chr(34) + "submit" + Chr(34) + " value = " + Chr(34) + "LED ausschalten" + Chr(34) + "></form>"
            Else
               Sendbuffer = "<input type=" + Chr(34) + "hidden" + Chr(34) + " name=" + Chr(34) + "LED" + Chr(34) + " value=" + Chr(34) + "0" + Chr(34) + ">"
               Sendbuffer = Sendbuffer + "<input Type = " + Chr(34) + "submit" + Chr(34) + " value = " + Chr(34) + "LED einschalten" + Chr(34) + "></form>"
            End If
         Elseif T = 8 Then
               Sendbuffer = "<br><br><hr><br>Bascom Software by Thomas Heldt<br><br>ATMEGA168-WIZ810MJ-Board by Thomas Heldt<br><br>Siehe auch Beitrag im "
         Elseif T = 9 Then
               Sendbuffer = "<a href=" + Chr(34) + "http://www.mikrocontroller.net/topic/81807" + Chr(34) + " target=" + Chr(34) + "_blank" + Chr(34)
         Else
               Sendbuffer = ">Mikrocontroller.net - Forum</a></font></center></body></html>"
         End If

         Sendsize = Len(Sendbuffer)

         Print "Bytes to send: " ; Sendsize

         Freesize2:
            Call Wiz5100_readvalue(&H4 , &H20)
            Freesize = Wert
            Shift , Freesize , Left , 8
            Call Wiz5100_readvalue(&H4 , &H21)
            Freesize = Freesize + Wert

            If Freesize < Sendsize Then
               Print "Freesize!!!"
               Goto Freesize2
            End If

         Call Wiz5100_readvalue(&H4 , &H24)
         Tx_wr = Wert
         Shift , Tx_wr , Left , 8
         Call Wiz5100_readvalue(&H4 , &H25)
         Tx_wr = Tx_wr + Wert

         Startpos = Tx_wr

         Offset = Tx_wr And &H7FF

         Startadress = &H4000 + Offset

         Offsend = Offset + Sendsize

         Pointer_tx = Startadress

         If Offsend > &H800 Then
            Print "Send action 2"

            Uppersize = &H800 - Offset

            For X = 1 To Uppersize
               Highbyte = High(pointer_tx)
               Lowbyte = Low(pointer_tx)
               Tmp_str = Mid(Sendbuffer , X , 1)
               Tmp_value = Asc(tmp_str)
               Call Wiz5100_writevalue(highbyte , Lowbyte , Tmp_value)
               Incr Pointer_tx
               Print Chr(tmp_value);
            Next X

            Pointer_tx = &H4000
            Incr Uppersize

            For X = Uppersize To Sendsize
               Highbyte = High(pointer_tx)
               Lowbyte = Low(pointer_tx)
               Tmp_str = Mid(Sendbuffer , X , 1)
               Tmp_value = Asc(tmp_str)
               Call Wiz5100_writevalue(highbyte , Lowbyte , Tmp_value)
               Incr Pointer_tx
               Print Chr(tmp_value);
            Next X
         Else
            Print "Send action 1"

            For X = 1 To Sendsize
               Highbyte = High(pointer_tx)
               Lowbyte = Low(pointer_tx)
               Tmp_str = Mid(Sendbuffer , X , 1)
               Tmp_value = Asc(tmp_str)
               Call Wiz5100_writevalue(highbyte , Lowbyte , Tmp_value)
               Incr Pointer_tx
               Print Chr(tmp_value);
            Next X
         End If

         Print ""
         Print "Get TX POINTER"

         Call Wiz5100_readvalue(&H4 , &H24)
         Startpos = Wert
         Shift , Startpos , Left , 8
         Call Wiz5100_readvalue(&H4 , &H25)
         Startpos = Startpos + Wert

         Glaenge = Startpos + Sendsize

         Print "Set SIZE to SEND"

         'Send Site
         Highbyte = High(glaenge)
         Call Wiz5100_writevalue(&H4 , &H24 , Highbyte)

         Lowbyte = Low(glaenge)
         Call Wiz5100_writevalue(&H4 , &H25 , Lowbyte)

         Print "Set SEND Flag"

        'Set SEND flag
         Call Wiz5100_writevalue(&H4 , &H1 , &H20)

      Next T

      Print "Set DISCON Flag"

      'Set DISCON flag
      Call Wiz5100_writevalue(&H4 , &H1 , &H8)
   End If
End Sub

'Receive data
Sub Wiz5100_receive()
   Local I As Word , Top As Word , Bytes_rcv As Word        ', Ssite As Byte , Found As Byte
   Local Headerend As String * 4

   Print "Receive Header"

   Call Wiz5100_readvalue(&H4 , &H26)
   Bytes_rcv = Wert
   Shift , Bytes_rcv , Left , 8
   Call Wiz5100_readvalue(&H4 , &H27)
   Bytes_rcv = Bytes_rcv + Wert

   'Clear Receivebuffer
   Receivebuffer = ""

   If Bytes_rcv > 0 Then
      Bytes_rcv = Bytes_rcv - 1

      For I = 0 To Bytes_rcv
          Top = I + &H6000
          Highbyte = High(top)
          Lowbyte = Low(top)
          Call Wiz5100_readvalue(highbyte , Lowbyte)
          Receivebuffer = Receivebuffer + Chr(wert)
      Next

      Print Receivebuffer

      'Authentification active and correct?
      If Auth = 1 Then
         Tmp_auth = Base64enc(userpass)

         If Instr(receivebuffer , Tmp_auth) > 0 Then
            'Authentification found and correct
            Authentification = 0
         Else
            'Authentification not found or not correct
            Authentification = 1
            Call Wiz5100_send()
         End If
      End If

      'Switch LED only if we get a correct POST Header
      If Instr(Receivebuffer , "LED=0") > 0 Or Instr(Receivebuffer , "LED=1") > 0 And Instr(Receivebuffer , "GET /") = 0 And Instr(Receivebuffer , "?/LED") = 0 Then
         If Instr(Receivebuffer , "LED=0") > 0 Then
            Reset Led
         Elseif Instr(Receivebuffer , "LED=1") > 0 Then
            Set Led
         End If

         'Set RECV flag
         Call Wiz5100_writevalue(&H4 , &H1 , &H40)

         'Send website
         Call Wiz5100_send()
     End If

     Headerend = Chr(13) + Chr(10) + Chr(13) + Chr(10)

     If Instr(Receivebuffer , Headerend) > 0 Then
         'Set RECV flag
         Call Wiz5100_writevalue(&H4 , &H1 , &H40)

         'Send website
         Call Wiz5100_send()
     End If
   End If
End Sub

Sub Wiz5100_init
   'Reset
   Call Wiz5100_writevalue(&H0 , &H0 , &H80)

   Waitms 500

   'Gateway IP adress
   Call Wiz5100_writevalue(&H0 , &H1 , Gw[1])
   Call Wiz5100_writevalue(&H0 , &H2 , Gw[2])
   Call Wiz5100_writevalue(&H0 , &H3 , Gw[3])
   Call Wiz5100_writevalue(&H0 , &H4 , Gw[4])

   'Subnetmask
   Call Wiz5100_writevalue(&H0 , &H5 , Sn[1])
   Call Wiz5100_writevalue(&H0 , &H6 , Sn[2])
   Call Wiz5100_writevalue(&H0 , &H7 , Sn[3])
   Call Wiz5100_writevalue(&H0 , &H8 , Sn[4])

   'MAC
   Call Wiz5100_writevalue(&H0 , &H9 , Mac[1])
   Call Wiz5100_writevalue(&H0 , &HA , Mac[2])
   Call Wiz5100_writevalue(&H0 , &HB , Mac[3])
   Call Wiz5100_writevalue(&H0 , &HC , Mac[4])
   Call Wiz5100_writevalue(&H0 , &HD , Mac[5])
   Call Wiz5100_writevalue(&H0 , &HE , Mac[6])

   'Own IP adress
   Call Wiz5100_writevalue(&H0 , &HF , Ip[1])
   Call Wiz5100_writevalue(&H0 , &H10 , Ip[2])
   Call Wiz5100_writevalue(&H0 , &H11 , Ip[3])
   Call Wiz5100_writevalue(&H0 , &H12 , Ip[4])

   'Initialize socket 0 with TCP on port 80
   Call Wiz5100_writevalue(&H4 , &H0 , &H1 )                'TCP

   Call Wiz5100_writevalue(&H4 , &H4 , &H0 )                'Port 80 HTTP
   Call Wiz5100_writevalue(&H4 , &H5 , &H50 )

   Call Wiz5100_writevalue(&H4 , &H1 , &H1)                 'OPEN
   Call Wiz5100_writevalue(&H4 , &H1 , &H2)                 'LISTEN

   End Sub

Sub Wiz5100_writevalue(byval Highbyte As Byte , Byval Lowbyte As Byte , Byval Value As Byte)

   Reset Wiz5100_cs
   A(1) = Wiz5100_opcode_write
   Spiout A(1) , 1
   A(1) = Highbyte
   Spiout A(1) , 1
   A(1) = Lowbyte
   Spiout A(1) , 1
   Spiout Value , 1
   Set Wiz5100_cs

End Sub

Sub Wiz5100_readvalue(byval Highbyte As Byte , Byval Lowbyte As Byte)

   Reset Wiz5100_cs
   A(1) = Wiz5100_opcode_read
   Spiout A(1) , 1
   A(1) = Highbyte
   Spiout A(1) , 1
   A(1) = Lowbyte
   Spiout A(1) , 1
   Spiin Wert , 1
   Set Wiz5100_cs

End Sub